@@ -1,6 +1,6 @@
 #! /usr/bin/env python
 """
-Query-based sampler for ProbLog 2.1
+Query-based sampler for ProbLog 2.1+
 ===================================
 
   Concept:
@@ -29,7 +29,7 @@ Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at
 
-     http://www.apache.org/licenses/LICENSE-2.0
+     https://www.apache.org/licenses/LICENSE-2.0
 
 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
